/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: gn/acc.c
    Version 0.6
    
    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "gn.h"

static int	match_select();



/*
 * chkcache( ip)  checks to see that the selector string sent by the client
 * is a valid one, i.e. that the selector exists in a cache file in the 
 * designated directory. It returns the (local not gopher) type of the item 
 * which ip->selector refers to.
 */

int
chkcache( ip)
Item	*ip;
{
	FILE	*fp,
		*fopen();
	char	*cp,
		*cachesel,
		linebuf[PATHLEN];

	Item	citem;

	if ( *(ip->cachepath) == '\0')
		/*
		 * root directory is the unique item not contained IN
                 * a cache file.
                 */
		return ROOT;

	if ( (fp = fopen( ip->cachepath, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open cache file (chkcache)", ip->cachepath);
		exit( 2);
	}

	while ( fgets( linebuf, PATHLEN, fp)) {
		cachesel = strchr( linebuf, '\t');
		cachesel++;
		cp = strchr( cachesel, '\t');
		*cp = '\0';
		parse_selector( &citem, cachesel, PARTIAL);

		if ( match_select( &citem, ip)) {
			switch (*(ip->gtype)) {
				case '0':
				case '4':
					return TEXT;
				case '1':
					if ( streq( ip->gtype, "1form"))
						return FORM;
					if ( streq( ip->gtype, "1choose"))
						return CHOOSE;
					return DIR;
				case '5':
				case '9':
				case 's':
				case 'I':
					return BIN;
				case '7':
					if ( streq( ip->gtype, "7form"))
						return FORM;
					return SEARCH;
				case 'e':
					return EXEC;
				case 'R':
					return RANGE;
				case 'i':
					return COMMENT;
				default:
					return UNKNOWN;
			}
		}
	}
	writelog( "Bad selector: ", ip->selector);
	return DENIED;
}

static int
match_select( ipc, ipr)
Item	*ipc,
	*ipr;
{

	/*
	 * Check that ipc (from .cache) and ipr have the same filepath
	 * and the same gtype.  One exception is the gytpe for
	 * ipc can be "1s" (searchable directory) and for ipr
	 * it can be "7g" (search).  Another "match" we want to 
	 * allow is "1form" and "7form".
	 */

	if ( ! streq( ipc->filepath, ipr->filepath))
		return FALSE;
	if ( streq( ipc->gtype, ipr->gtype))
		return TRUE;
	if ( streq(ipr->gtype, "7g") && streq(ipc->gtype, "1s"))
		return TRUE;
	if ( streq(ipr->gtype, "7form") && (streq(ipc->gtype, "1form")))
		return TRUE;
	return ( FALSE );
}



/*
 * chkaccess( ip, log) checks whether the client's IP address is in the allowed
 * list (either the access file for each directory if accesstype=DIRCHK or
 * the access file in the root directory if accesstype=ROOTDIR).  If allowed
 * it returns TRUE, if not FALSE.  If accesstype=FREE, it just returns TRUE.
 * If log is TRUE and access is denied a note of the failure is placed in
 * the log file.
 */

int
chkaccess( ip, log)
Item	*ip;
int	log;
{
	FILE	*fp;
	int	len,
		notflag,
		found = FALSE;
	char	*cp,
		*rptr,
		linebuf[PATHLEN],
		accessfile[PATHLEN];

		struct stat stat_buf;

	switch ( accesstype) {
		case FREE:
			return TRUE;
		case ROOTCHK:
			sprintf( accessfile, "%s/%s", ip->accdir, afname);
			break;
		case DIRCHK:

		/* Look for access file in current directory, and if
		   not found work our way up to rootdir looking for it */

			strcpy( accessfile, ip->accdir);
			rptr = accessfile + strlen( rootdir);
			cp = rptr + 1;
			while ( cp > rptr ) {
				cp = strrchr( accessfile, '/');
				strcpy( (cp + 1), afname);
				if ( stat( accessfile, &stat_buf) == 0 ) {
					found = TRUE;
					break;
				}
				*cp = '\0';
			}
			if ( found == FALSE ){	/* No access file so allow */
				return TRUE;
			}
			break;
		default:
			senderr( "Unknown access error", ip);
	}

	if ((fp = fopen( accessfile, "r")) == (FILE *)NULL ) {
		senderr2( "Can't open file", accessfile);
		exit( 2);
	}

	while ( fgets( linebuf, PATHLEN, fp)) {
		chop(linebuf);
		cp = linebuf;

		if ( notflag = ( *cp == '!'))
			cp++;
		len = strlen( cp);

		if ( (len == 0) || (*cp == '#') )
			continue;

		strlower( cp);
		if ( strncmp( cp, remaddr, len) == 0 ||
		strncasecmp( cp, remotehost, len) == 0 ||
			wild_match( remotehost, cp)) {
			if ( notflag)
				break;
			else {
				fclose( fp);
				return TRUE;
			}
		}
	}
	fclose( fp);
	if ( log)
		writelog("Access denied request was: ", ip->selector);
	return FALSE;
}
