/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: gn/init.c
    Version 0.6

    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include "gn.h"


extern char *optarg;
extern int optind;

char	rootdir[PATHLEN],
	cfname[PATHLEN],
	afname[PATHLEN],
	myhost[BUFSIZE],
	remotehost[BUFSIZE],
	remaddr[PATHLEN];

int	accesstype = FREE;

static void	locklog(),
		unlocklog();

static char	logfile[PATHLEN];

void
gn_init( argc, argv)
int	argc;
char	*argv[];
{
	int	c,
		errflg = 0;

	alarm( TIMEOUT);  /* Timeout after TIMEOUT seconds */
	getmyhost( myhost);
	strcpy( rootdir, ROOT_DIR);
	strcpy( cfname, CACHEFNAME);
	strcpy( afname, ACCESSFNAME);
	logfile[0] = '\0';

	while ((c = getopt(argc, argv, "aAL:c:g:")) != -1) {
		switch ((char) c) {
			case 'a':
				accesstype = ROOTCHK;
				break;
			case 'A':
				accesstype = DIRCHK;
				break;
			case 'L':
				strcpy( logfile, optarg);
				break;
			case 'c':
				strcpy( cfname, optarg);
				break;
			case 'g':
				strcpy( afname, optarg);
				break;
			case '?':
				errflg++;
		}
	}

	if (errflg) {
		fprintf( stderr, 
	"Usage: %s [-L logfile] [-a | -A] [-c cache] [rootdir]\n",
		argv[0]);
		exit (2);
	}
	if ( argv[optind] )
		strcpy( rootdir, argv[optind]);

	get_remote_info( remaddr, remotehost);
}


void
writelog( msg, msg2)
char	*msg,
	*msg2;
{
	FILE	*logfp = NULL;
	struct flock	lck;

	time_t		clock;
	char            *cp,
			*ctime();

	if ( *logfile != '\0') {
		if ( (logfp = fopen( logfile, "a")) == NULL ) {
			fprintf( stderr, "Can't open logfile: %s\n", logfile);
			exit( 2);
		}

		time(&clock);
		cp = ctime(&clock);

		locklog( &lck, fileno(logfp) );
		fseek( logfp, 0L, 2);   
		fprintf(logfp, "%s: %s (%s): %s",  remotehost, msg, msg2, cp);
		unlocklog( &lck, fileno(logfp) );   
		fclose( logfp);
	}
}


static void
locklog(lck, fd)
struct flock	*lck;
int	fd;
{
	lck->l_type = F_WRLCK;
	lck->l_whence = SEEK_SET;
	lck->l_start = 0L;
	lck->l_len = 0L;
	fcntl(fd, F_SETLKW, lck);
}

static void
unlocklog( lck, fd)
struct flock	*lck;
int	fd;
{
	lck->l_type = F_UNLCK;
	fcntl( fd, F_SETLKW, lck);
}

