/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: gn/pselect.c
    Version 0.6
    
    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/

#include <stdio.h>
#include <string.h>
#include "gn.h"



static	void	entergtype(),
		enterextra(),
		enterfilepath(),
		entercachepath(),
		enteraccdir(),
		enterrange();

static	char	explicit_cache[PATHLEN],
		*pathptr;

static int	hasparens;


/*
 * parse_selector( ip, sel, full) fills in entries of the Item struct pointed
 * to by ip.  It destroys the line pointed to by sel.  If full == TRUE it
 * does a full parse needed for selectors received from client.  If full ==
 * FALSE it does a partial providing what is needed to parse selectors which
 * are in our cache files.  Here's the struct:
 *
 * typedef struct Item {
 *	char	selector[PATHLEN],	* The original selector
 *		gtype[PATHLEN],		
 *					 * Typically gopher type 0,1, etc.
 *					 * but really everything up to first
 *                                       * '/' in the selector
 *
 * 		filepath[PATHLEN],	* Complete pathname of file  *
 * 		cachepath[PATHLEN],	* Complete pathname of cache which
 *					* which contains this item 
 * 		accdir[PATHLEN],	* path of access file directory *
 *		extra[PATHLEN],		* Stuff after the last tab *
 *		args[PATHLEN];		* Args for exec *
 *
 *	long	range_start,		* Start and end of text file range *
 * 		range_end;
 *
 *	int	compressed;		* True if file is compressed *
 * } Item;
 *
 *
 *   A selector looks like
 *   0/dir1/dir2/file(explicit_cache) or
 *   1/dir/dir/the_dir or 1/dir/dir/the_dir/cachfile(explicit_cache) or
 *   exec:args:/dir1/dir2/script(explicit_cache)  or 
 *   7/dir1/dir2/script <tab> searchterm
 * The (explicit_cache) is optional in all cases.  If present it means 
 * the cachefile containing 0/dir1/dir2/file  has name given by the 
 * contents of explicit_cache appended.  Note that for menus (directories) 
 * the syntax of the selector is different depending on whether 
 * (explicit_cache) is present.  If absent the selector includes
 * the pathname to the directory, e.g. /dir1/dir2/the_dir. But if 
 * (explicit_cache) is present, the pathname is to the cachefile in
 * the directory, e.g. /dir1/dir2/the_dir/cachefile.
 */


void
parse_selector( ip, select, full)
Item	*ip;
char	*select;
int	full;
{
	register
	char	*cp;

	strcpy( ip->selector, select);
	/* Change tabs to spaces in ip->selector - no tabs in logfile */

	if ( (cp = strchr( ip->selector, '\t')) != NULL)
		while ( *cp) {
			*cp = ( (*cp == '\t') ? ' ' : *cp );
	  		cp++;
		}

	if ( ((pathptr = strchr( select, '/')) == NULL) ||
			streq( select, "1/") ) {
		/*
		 * It's the root directory.  Root is a special case 
		 * Handle it here.
		 */
		strcpy( ip->gtype, "1");
		sprintf( ip->filepath, "%s/%s", rootdir, cfname);
		sprintf( ip->accdir, "%s/", rootdir);
		*(ip->cachepath) = '\0';  /* root not contained IN a cache */
		return;
	}

	/* If select ends in <tab>stuff, put stuff in ip->extra and null tab */
	enterextra( ip, select);

	entergtype( ip, select);

	/* If the selector ends in "(stuff)" put stuff in explicit_cache 
	 * and remove parens and their contents from select.
	 */

	hasparens = FALSE;
	if ( (cp = strrchr( pathptr, '(')) != NULL) {
		hasparens = TRUE;
		*cp++ = '\0';
		strcpy( explicit_cache, cp);
		if ( (cp = strchr( explicit_cache, ')')) == NULL) {
			senderr( "Bad selector", ip);
			exit( 2);
		}
		*cp = '\0';
	}

	enterfilepath( ip);
	if ( full == FULL) {
		entercachepath( ip);
		enteraccdir( ip);
		enterrange( ip);
		ip->compressed = ip->gtype[1] == 'Z';
	}
}

static void
enterextra( ip, sel)
Item	*ip;
char	*sel;
{
	register char	*cp;

	/* if selector ends in <tab>stuff, put "stuff" in extra */

	if ( (cp = strrchr( sel, '\t')) != NULL) {
		*cp++ = '\0';
		strcpy( ip->extra, cp);

	} else
		*(ip->extra) = '\0';
}


static void
entergtype( ip, sel)
Item	*ip;
char	*sel;
{
	register char	*cp,
			*argp;

	/* gtype is everything up to the first ':' or '/' in the selector */
	strcpy( ip->gtype, sel);
	if ( (argp = strchr( ip->gtype, ':')) != NULL) {
		cp = strrchr( ip->gtype, ':');
		*cp = '\0';

	/* Extract args from gtype of the form exec:args: or 7form:args:  */
		*argp++ = '\0';
		strcpy( ip->args, argp);

	} else {
		*(ip->args) = '\0';
		cp = strchr( ip->gtype, '/');
		*cp = '\0';
	}
}

static void
enterfilepath( ip)
Item	*ip;
{
	register char	*cp;

	/*
	 * In ip->filepath put complete path, i.e. rootdir +
	 * everything in selector from first '/' up to tab or \n or '('
	 */

	strcpy( ip->filepath, rootdir);
	cp = ip->filepath;
	strcat( cp, pathptr);

	/* Remove a '/' at end of filepath, if it's there */
	while (*cp)
		cp++;
	if ( *--cp == '/' )  
		*cp = '\0';

	/*
	 * For a directory (type 1) we want filepath to be the name of the
	 * cache file containing the directory.  If hasparens == TRUE
	 * this should already be the case.  Also we don't do it when gtype
	 * is "1form". Otherwise we tack on cfname.  We also tack it on for
	 * for grep search (type 7g).
	 */

	if ( *(ip->gtype) == '1') {
		if ( hasparens || streq( ip->gtype, "1form"))
			return; 
		if ( *(ip->gtype +1) ==  'm') {
			strcat( ip->filepath, ".");
			strcat( ip->filepath, cfname);
		}
		else {
			strcat( ip->filepath, "/");
			strcat( ip->filepath, cfname);
		}
	}

	if ( streq( ip->gtype, "7g") && (hasparens == FALSE)) {
		strcat( ip->filepath, "/");
		strcat( ip->filepath, cfname);
	}

}



static void
entercachepath( ip)
Item	*ip;
{
	register char	*cp;

	/*
	 * In ip->cachepath put complete path to the cachefile containing
	 * this item.  If hasparens is true the relative path to the cachefile
	 * is already in explicit_cache, otherwise the cachefile is always 
	 * assumed to be in the directory containing this item 
	 * (whether this is a file or directory) and assumed to have the
	 * name cfname.
	 *
	 * If it is a range type then gtype starts with 'R' and 
	 * the cache file is named
	 * path/basename.cache  else named path/.cache or has been given
	 * explicitly. 
	 */
	
	if ( hasparens) {
		strcpy( ip->cachepath, rootdir);
		strcat( ip->cachepath, explicit_cache);
		return;
	}
	if ( *(ip->gtype) == 'R')
		sprintf( ip->cachepath, "%s.%s", ip->filepath, cfname);
	else {
		strcpy( ip->cachepath, rootdir);
		strcat( ip->cachepath, pathptr);

		cp = strrchr( ip->cachepath, '/');
		strcpy( ++cp, cfname);
	}
}

static void
enteraccdir( ip)
Item	*ip;
{
	register char	*cp;

	/*
	 * In ip->accdir put complete path of directory containing the
	 * access file for this item.
	 */

	switch ( accesstype) {
		case FREE:
			*(ip->accdir) = '\0';
			return;

		case ROOTCHK:
			sprintf( ip->accdir, "%s/", rootdir);
			return;

		case DIRCHK:
			strcpy( ip->accdir, ip->filepath);
			cp = strrchr(ip->accdir, '/');
			*(++cp) = '\0';
	}
}



static void
enterrange( ip)
Item	*ip;
{
	register char	*cp,
			*cp2;
	long		atol();

	/* if gtype = Rr1-r2-1m/path it is a text file range from
	 * byte r1 to r2.  Put r1 in range_start and r2 in range_end
	 */

	if ( *(ip->gtype) == 'R' ) {
		if ( (cp = strchr( ip->gtype, '-')) == NULL) {
			senderr( "Malformed range type", ip);
			exit( 2);
		}
		*cp = '\0';
		ip->range_start = atol( ip->gtype + 1);
		*cp++ = '-';
		if ( (cp2 = strchr( cp, '-')) == NULL) {
			senderr( "Malformed range type", ip);
			exit( 2);
		}
		*cp2 = '\0';
		ip->range_end = atol( cp);
		*cp2 = '-';
	} else
		ip->range_start = ip->range_end = 0L;
}
