/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: gn/search.c
    Version 0.6
    
    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <string.h>
#include "gn.h"
#include "reg.h"

#define	ERROR	(-1)
#define MATCH	(0)
#define NOMATCH	(1)


/* make up for deficient stat.h's */

#ifndef S_ISDIR
#define S_ISDIR(mode) ((mode) & S_IFREG)
#endif


static int	found_match();
static void	dogrep(),
		range_search();

/*
 * sendsearch( ip)  Send the items from a "virtual cachefile" produced
 * running the search program ip->filepath.
 */

void
sendsearch(  ip)
Item	*ip;

{
	FILE	*fp;
	char	linebuf[PATHLEN];

	struct stat stat_buf;

	if ( *(ip->extra) == '\0') {
		senderr("Search string is empty!", ip);
		exit( 2);
	}

	switch( ip->gtype[1]) {
		case 'c':
			cache_search( ip);
			return;

		case 'm':
			range_search( ip);
			return;

		case 'g':
			dogrep( ip);
			return;
		case '\0':
			if ( (stat( ip->filepath, &stat_buf) == 0) && 
					(S_ISDIR( stat_buf.st_mode)) ) {
					/* It's a grep search */
				dogrep( ip);
				return;
			}
	}
	
	if ( (fp = safer_popen( ip->filepath, ip->extra, "r"))
					 == (FILE *) NULL ) {
		senderr2( "Can't exec", ip->filepath);
		exit( 2);
	}

	while ( fgets( linebuf, BUFSIZE, fp)) {
		chop( linebuf);
		if ( streq( linebuf, "."))
			strcat( linebuf, ".");  
			/* Double a '.' if on line by itself */
		strcat( linebuf, "\r\n");
		send_cache_line( linebuf);
	}	
	pclose( fp);
	send_end();
	writelog( "Search", ip->selector);
}

/*
 * range_search( ip)  This is type "7m",  we open the composite cachefile
 * basename..cache and also the composite file "basename".  For each
 * cachefile line search the corresponding range in the composite file
 * for lines matching the search term pointed to by ip->regptr.
 * When a match is found send the cachefile line.
 */

static void
range_search( ip)
Item	*ip;

{
	Item		curitem,
			*item_p;

	Cache_entry	cstruct,
			*cstructp;
	FILE	*fp,
		*cfp;

	struct regprog	*regp;

	char	cachepath[PATHLEN],
		cline[BUFSIZE],
		linebuf[BUFSIZE];

	item_p = &curitem;
	cstructp = &cstruct;

	if ( (regp = regcomp( strlower( ip->extra))) == NULL ) {
		senderr2( "Search term regular expression error: \n",
			ip->extra);
		exit( 2);
	}


	sprintf( cachepath, "%s.%s", ip->filepath, cfname);

	if ( (fp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open file", ip->filepath);
		exit( 2);
	}
	if ( (cfp = fopen( cachepath, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open file", cachepath);
		exit( 2);
	}

	while ( fgets( cline, BUFSIZE, cfp)) {
		parse_cache( cline, cstructp);
		parse_selector( item_p, cstructp->path, FULL);

		fseek( fp, item_p->range_start, 0);
		while ( fgets( linebuf, BUFSIZE, fp) && 
				(ftell( fp) <= item_p->range_end)) {
			chop( linebuf);
			if ( regfind( regp, strlower( linebuf)) ) {
				send_cache_line( cline);
				break;
			}
		}
	}	
	send_end();
	writelog( "Sent range search", ip->selector);
}







/* Do regexp search of directory pointed to by ip->filepath */

static void
dogrep( ip)
Item	*ip;
{
	FILE	*fp,
		*gfp;

	struct regprog	*regp;

	char	*cp,
		cachefile[PATHLEN],
		tmpbuf[PATHLEN],
		pathbuf[PATHLEN],
		linebuf[PATHLEN];

	
	if ( streq( ip->gtype, "7g") )
		strcpy(cachefile, ip->filepath);
	else
		sprintf( cachefile, "%s/%s", ip->filepath, cfname);

	if ( (fp = fopen( cachefile, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open", cachefile);
		exit( 2);
	}

	while ( fgets( linebuf, BUFSIZE, fp)) {
		strcpy( tmpbuf, rootdir);
		cp = strchr( linebuf, '\t');
		if ( *++cp != '0' )	/* Quit if not type 0 (file) */
			continue;
		strcat( tmpbuf, ++cp);
		cp = strchr( tmpbuf, '\t'); 
		*cp = '\0';
		if ( (cp = strrchr( tmpbuf, '(')) != (char *) NULL)
			*cp = '\0';
		cp = strchr( tmpbuf, '/');
		sprintf( pathbuf, "%s", cp);

                if ( (gfp = fopen( pathbuf, "r")) == (FILE *) NULL ) {
                        /* File is no longer there, keep going */
                        continue;
                }

		if ( (regp = regcomp( strlower(ip->extra))) == NULL ) {
			senderr2( "Search term regular expression error: \n",
					ip->extra);
			break;
		}		
		if ( found_match( regp, gfp) )
			send_cache_line( linebuf);

		fclose( gfp);
	}
	send_end();
}


static int
found_match( rp, fp)
struct regprog	*rp;
FILE	*fp;
{
	char linebuf[BUFSIZE];

	while ( fgets( linebuf, BUFSIZE, fp)) {
		strlower( linebuf);
		if ( regfind( rp, linebuf) )
			return TRUE;
	}
	return FALSE;
}

