/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: gn/send.c
    Version 0.6
    
    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/

#include <stdio.h>
#include <string.h>
#include "gn.h"

static char	*fieldp,
		newfields[BUFSIZE];
static void	send_form_line(),
		mknewfields();

/*
 * sendtext( ip)  Send the text file with Item pointer ip.
 */

void
sendtext( ip)
Item	*ip;

{
	FILE	*fp;
	char	linebuf[BUFSIZE],
		commandbuf[BUFSIZE];


	if ( ip->compressed) {
		sprintf( commandbuf, "%s %s", DECOMPRESS, ip->filepath);
		if ( (fp = popen( commandbuf, "r")) == (FILE *) NULL ) {
			senderr2( "Can't decompress", ip->filepath);
			exit( 2);
		}
	}
	else
		if ( (fp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
			senderr2( "Can't open file", ip->filepath);
			exit( 2);
		}

	while ( fgets( linebuf, BUFSIZE-2, fp)) {
		chop( linebuf);
		if ( streq( linebuf, "."))
			strcat( linebuf, ".");  
			/* Double a '.' if on line by itself */
		send_text_line( linebuf);
	}	
	send_end();
	writelog( "Sent text", ip->selector);

	if ( ip->compressed)
		pclose( fp);
	else
		fclose( fp);
}

/*
 * sendrange( ip)  Send a range from a textfile
 */

void
sendrange( ip)
Item	*ip;

{
	FILE	*fp;

	char	linebuf[BUFSIZE];

	if ( (fp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open file", ip->filepath);
		exit( 2);
	}
	fseek( fp, ip->range_start, 0);
	while ( fgets( linebuf, BUFSIZE, fp) && (ftell( fp) <= ip->range_end)) {
		chop( linebuf);
		if ( streq( linebuf, "."))
			strcat( linebuf, ".");  
			/* Double a '.' if on line by itself */
		send_text_line( linebuf);
	}	
	send_end();
	writelog( "Sent range", ip->selector);
}




/*
 * sendbin( ip)  Send a binary file.
 */

void
sendbin(  ip)
Item	*ip;

{
	FILE	*fp;
	char	commandbuf[PATHLEN];
	int	c;


	if ( ip->compressed) {
		sprintf( commandbuf, "%s %s", DECOMPRESS, ip->filepath);
		if ( (fp = popen( commandbuf, "r")) == (FILE *) NULL ) {
			senderr2( "Can't decompress", ip->filepath);
			exit( 2);
		}
	}
	else
		if ( (fp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
			senderr2( "Can't open file", ip->filepath);
			exit( 2);
		}
	
	while ( (c = fgetc( fp )) != EOF)
		fputc( c, stdout);
	writelog( "Sent binary", ip->selector);

	if ( ip->compressed)
		pclose( fp);
	else
		fclose( fp);
}

/*
 * sendcache( ip)  Send the contents of the cachefile
 * in the directory pointed to by selector.  ip->filepath is the complete
 * pathname of the directory.
 */

void
sendcache(  ip)
Item	*ip;

{
	FILE	*fp;

	char	cline[BUFSIZE];

	if ( (fp = fopen( ip->filepath, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open cache file (sendcache): ", ip->filepath);
		exit( 2);
	}
	
	while ( fgets( cline, BUFSIZE, fp))
		send_cache_line( cline);

	send_end();
	writelog( "Sent cache", ip->selector);
}		



/*
 * sendexec( ip)  Open pipe from "ip->filepath" command
 * and send output.
 */

void
sendexec(  ip)
Item	*ip;
{
	FILE	*fp;
	register int	c;
	char	linebuf[PATHLEN];


	if ( (fp = safer_popen( ip->filepath, ip->args, "r"))
				 == (FILE *) NULL ) {
		senderr2( "Can't exec", ip->filepath);
		exit( 2);
	}

	switch (*(ip->gtype + 4)) {  /* the char after "exec" in gtype */
		case '1':
			while ( fgets( linebuf, BUFSIZE, fp))
				send_cache_line( linebuf);
			break;
		case '5':
		case '9':
		case 's':
		case 'I':
			while ( (c = fgetc( fp )) != EOF)
				fputc( c, stdout);
			pclose( fp);
			writelog( "Exec binary", ip->selector);
			return;
		case '0':
		case '4':
		case ':':
		default:
			while ( fgets( linebuf, BUFSIZE, fp))
				send_text_line( linebuf);
			break;
	}

	pclose( fp);
	send_end();
	writelog( "Exec", ip->selector);
}

void
sendform( ip)
Item	*ip;
{
	FILE	*fpform;
	char	formfile[PATHLEN],
		linebuf[BUFSIZE];


	strcpy( formfile, ip->filepath);
	strcat( formfile, ".cache");

	if ( (fpform = fopen( formfile, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open cache file (sendform): ", formfile);
		exit( 2);
	}
	if ( streq( ip->gtype, "7form")) {
		mknewfields( ip->args, ip->extra);
	} else if ( streq( ip->gtype, "1form"))
		strcpy( newfields, ip->args);	/* put in default fields */
	else
		senderr( "Something went wrong in send_form", ip);

	fieldp = newfields;

	while ( fgets( linebuf, BUFSIZE, fpform)) {
		if ( *linebuf == '\r' )
			continue;
		send_form_line( ip,  linebuf);
	}
        fclose( fpform);
        send_end();
        writelog( "Form", ip->selector);
}	


void
sendchoose( ip)
Item	*ip;
{
	FILE	*fpchoose;
	char	*cp,
		choosefile[PATHLEN],
		cacheline[BUFSIZE],
		linebuf[BUFSIZE];


	strcpy( choosefile, ip->filepath);

	if ( (fpchoose = fopen( choosefile, "r")) == (FILE *) NULL ) {
		senderr2( "Can't open cache file (sendchoose): ", choosefile);
		exit( 2);
	}

	while ( fgets( linebuf, BUFSIZE, fpchoose)) {
		if ( *linebuf == '\r' )
			continue;
		if ( (cp = strchr( linebuf, '\t')) ==  NULL ) {
			senderr( "Corrupt cacheline (sendchoose)", ip);
			exit( 2);
		}
		
		strcpy( cacheline, linebuf);
		*cp++ = '\0';

		if ( strncmp( cp, "1form", 5 ) == 0) {
			if ( (cp = strchr( cp, '/')) ==  NULL ) {
				senderr( "Corrupt chooseline (sendchoose)", ip);
				exit( 2);
			}
			mknewfields( ip->args, linebuf + 1);

			sprintf(cacheline, "%s\t1form:%s:%s", linebuf,
				newfields, cp);
		}
		send_cache_line( cacheline);
	}

        send_end();
        writelog( "Choice", ip->selector);
        fclose( fpchoose);
}	




static void
send_form_line( ip, formline)
Item	*ip;
char	*formline;
{
	static int	itemnum = 0;
	char	*cp,
		*cp2,
		*type,
		currentfield[BUFSIZE],
		cacheline[BUFSIZE];

	strcpy(cacheline, formline); /* Save for use as default */

	if ( (cp = strchr( formline, '\t')) ==  NULL ) {
		senderr( "Corrupt formline (sendform)", ip);
		exit( 2);
	}
	*cp++ = '\0';

	if ( strncmp( cp, "exec", 4 ) == 0) {
		if ( (cp = strchr( cp, '/')) ==  NULL ) {
			senderr( "Corrupt formline (sendform)", ip);
			exit( 2);
		}
		sprintf( cacheline, "%s\texec%c:%s:%s",
				formline, *formline, newfields, cp);
	}
	else if ( (strncmp( cp, "7form", 5 ) == 0) ||
			(strncmp( cp, "1choose", 7 ) == 0)) {
		if ( *cp == '7')
			type = "7form";
		else
			type = "1choose";

		if ( (cp = strchr( cp, '/')) ==  NULL ) {
			senderr( "Corrupt formline (sendform)", ip);
			exit( 2);
		}
		itemnum++;
		cp2 = currentfield;
		if ( *fieldp )
			fieldp++;

		while ( *fieldp && *fieldp != '"')
			*cp2++ = *fieldp++;
		*cp2 = '\0';

		if ( *fieldp )
			fieldp++;

		while ( *fieldp && *fieldp != '"')
			*fieldp++;
		
		if ( *currentfield)
			sprintf(cacheline, "%s [%s]\t%s:%d %s:%s", formline,
				currentfield, type, itemnum, newfields, cp);
		else
			sprintf(cacheline, "%s\t%s:%d %s:%s", formline, type,
				itemnum, newfields, cp);
	}

	send_cache_line( cacheline);
}


static void 
mknewfields( oldfields, replacement)
char	*oldfields,
	*replacement;
{
	char	*cp,
		*cp2,
		*cp3,
		fields[BUFSIZE];
	int	fieldnum,
		i;

		strcpy( fields, oldfields);
		/*
		 * fields is a list separated by spaces.  The first element
		 * being the number (in ascii) of the field for this item
		 * and the remainder being quoted strings in order, containing
		 * the fields.  The list looks like
		 *   3 "field 1" "field 2" "field 3" "field 4"
		 */

		/*
		 * At this point we copy the first (fieldnum - 1) fields 
		 * from fields to newfields, then copy the fieldnum field
		 * from ip->extra, then copy the remaining fields.  The
		 * 1st arg of the list (the number of this item) is *not*
		 * in newfields.  It will be added in send_form().  We want
		 * newfields to be 
		 *    "field 1" "field 3" "newfield 3" "field 4"
		 */
	
		cp = strchr( fields, '"');
		strcpy( newfields, cp);
		fieldnum = atoi( fields);
		cp2 = newfields;
		
		i = 1;
		while ( i < fieldnum) {
			while ( *cp && *cp != '"') {
				cp++;
				cp2++;
			}
			if ( *cp == '\0') 
				break;
			cp++;
			cp2++;
			i++;
			while ( *cp && *cp != '"') {
				cp++;
				cp2++;
			}
			if ( *cp == '\0') 
				break;
			cp++;
			cp2++;
			while ( *cp && *cp != '"') {
				cp++;
				cp2++;
			}
			if ( *cp == '\0') 
				break;
		}
		if ( *cp == '\0') {
			senderr2( "Bad field list in form selector", "");
			exit( 2);
		}

		cp2++;

		cp3 = replacement;
		while ( *cp3 ) {	/* Replace field with new one */
			if ( *cp3 == '"' )
				*cp3 = ' ';
			*cp2++ = *cp3++;
		}

		cp++;
		while ( *cp && *cp != '"' )	
			cp++;
		if ( *cp == '\0') {
			senderr2( "Bad field list in form selector", "");
			exit( 2);
		}

		strcpy( cp2, cp); /* copy remaining fields */
}
	
