/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: gn/util.c
    Version 0.6

    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <netdb.h>
#include "gn.h"


void
senderr( msg, ip)
char	*msg;
Item	*ip;
{
	fprintf( stdout, "0Server error: %s\t\terrorhost\t1\r\n.\r\n", msg);
	writelog( msg, ip->selector);
}

void
senderr2( msg, msg2)
char	*msg,
	*msg2;
{
	if ( msg2 == NULL)
		msg2 = "";
	fprintf( stdout, "0Server error: %s\t\terrorhost\t1\r\n.\r\n",
					msg);
	writelog( msg, msg2);
}


void
get_remote_info( ipnum, hostp)
char	*ipnum,
	*hostp;
{
	char			*inet_ntoa();
	struct sockaddr_in	saddr;
	struct hostent		*hostentp = NULL;
	int			size;

#ifdef NONET
	*ipnum = *hostp = '\0';
	return;
#endif
	size = sizeof(saddr);
	getpeername(fileno(stdin), &saddr, &size);
	strcpy(ipnum, inet_ntoa(saddr.sin_addr));

	hostentp = gethostbyaddr((char *) &saddr.sin_addr,
			sizeof (saddr.sin_addr.s_addr), AF_INET);

		if (hostentp == NULL)
			strcpy( hostp, ipnum);
		else {
			strcpy( hostp, hostentp->h_name);
			strlower( hostp);
		}
}

void
getmyhost( buf)
char	*buf;
{
	static char dns_name[MAXHOSTNAMELEN];
	struct hostent *hostp;

	if (gethostname(dns_name, MAXHOSTNAMELEN) != 0) {
	fprintf(stderr, "Can't get the name of this host.\n");
		exit( 2);
	}

	hostp = gethostbyname( dns_name);

	if ( hostp != NULL )
		strncpy( buf, hostp->h_name, MAXHOSTNAMELEN );
	else
		strcpy( buf, dns_name);
}


/* chop( line)  Cut out CRLF at end of line */

void
chop( line)
char *line;
{
	char	*p;

	if ( *line == '\0')
		return;
	if ( (p = strchr( line, '\n')) == (char *) NULL )
		return;
	if ( *--p != '\r')
		p++;
	*p = '\0';
}

/*
 *  wild_match
 *
 *  String equality routine, including matching the '*' character.
 */

/*
 *  Borrowed from the ANU News sources V6.1b3 newsrtl.c.  Original sources
 *  Copyright 1989 by Geoff Huston.
 *
 */

int wild_match(l,p)
    char *l,
         *p;
{
    if (!*l) {
        if (!*p) return(1);
        else if (*p == '*') return(wild_match(l,p+1));
        else return(0);
        }
    if (*p == '*') {
        while (!wild_match(l,p+1)) {
            l++;
            if (!*l) {
                if (!*(p+1)) return(1);
                else return(0);
                }
            }
        return(1);
        }
    if ((*p == '%') || (*p == '?')) return(wild_match(l+1,p+1));
    if (*p == '^') return((*l == *(p+1)) && wild_match(l+1,p+2));
    return((*l == *p) && wild_match(l+1,p+1));
}

/*
 *  Make string all lower case.
 */


char *
strlower( st)
char	*st;
{
	register char	*cp;

	cp = st;
	while ( *cp) {
		*cp =  (isupper(*cp) ? *cp - 'A' + 'a' : *cp );
		cp++;
	}
	return (st);
}

FILE *
safer_popen( command, args, type)
char	*command,
	*args,
	*type;
{
	register char *cp;
	char	buf[PATHLEN];

	cp = args;
	while ( *cp != '\0') {
		switch (*cp) {
			case	';':
			case	'`':
			case	'|':
			case	'\'':
			case	'*':
			case	'?':
			case	'-':
			case	'~':
			case	'>':
			case	'^':
			case	'[':
			case	']':
			case	'(':
			case	')':
			case	'{':
			case	'}':
			case	'$':
			case	'=':
			case	'!':
			case	'/':
			case	'\\':
				*cp = '_';
		}
		cp++;
	}
	if ( *args != '\0')
		sprintf( buf, "%s %s\n", command, args);
	else
		strcpy( buf, command);

	return (popen( buf, type));
}