/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: mkcache/form.c
    Version 0.6
    
    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "mkcache.h"
#include "reg.h"

static void	dochoice(),
		writelink();

static 	FILE	*fcachefp,
		*choosefp;

void
doform( fname, cpath, menufp, cfp)
char	*fname,
	*cpath;

FILE	*menufp,
	*cfp;
{

	int	choosenum = -1;

	char	*cp,
		*cp2,
		*text,
		exectype = '0',
		argtmp[MAXLEN],
		arglist[MAXLEN],
		lname[MAXLEN],
		lpath[MAXLEN],
		ltype[MAXLEN],
		lhost[MAXLEN],
		lport[MAXLEN],
		formname[MAXLEN],
		pathline[MAXLEN],
		fcpath[MAXLEN],
		formpath[MAXLEN],
		*bufp,
		mbuf[MAXLEN];

	strcpy( formname, fname);
	bufp = get_next_line( mbuf, MAXLEN, menufp);
	
	if ( strncmp(bufp, "Path=", 5) != 0) {
		fprintf( stderr, "Bad path line for form: %s\n", bufp);
		exit( 2);
	}
	cp = strchr( bufp, '=');
	strcpy( pathline, ++cp);
		
	if ( ( cp = strchr( pathline, '/')) == NULL) {
		fprintf( stderr, "Bad path for form: %s\n", pathline);
		exit( 2);
	}
	strcpy( formpath, cp);
	cp2 = strrchr( formpath, '/');


	strcpy( fcpath, cpath);
	cp = strrchr( fcpath, '/');
	strcpy( cp, cp2);
	strcat( fcpath, cfname);

	if ( (fcachefp = fopen( fcpath, "w")) == (FILE *) NULL ) {
		fprintf( stderr, "Can't open %s\n", fcpath);
		exit( 2);
	}

	printf( "Writing form cache file %s\n", fcpath);

	while ( bufp = get_next_line( mbuf, MAXLEN, menufp)) {
		if ( strncmp( bufp, "Endform", 7) == 0 ) {
			cp = strchr( pathline, '/');
			fprintf( cfp, "1%s\t1form:%s:%s\t%s\t%s\r\n",
				formname, arglist, cp, 
					myhost, defaultport);

			if ( *lname != '\0')
 				writelink( lname, lpath, ltype, lhost, lport);
			*arglist = '\0';
			fclose( fcachefp);
			return;
		}
		cp = strchr( bufp, '=');
		*cp++ = '\0';
		text = cp;
	
		if ( streq( bufp, "Field")) {
			if ( lname[0] != '\0') 
				writelink( lname, lpath, ltype, lhost, lport);
			if ( ( cp = strchr( text, '\t')) != NULL ) {
				*cp++ = '\0';
				sprintf( argtmp, "\"%s\" ", cp);
			}
			else
				strcpy( argtmp, "\"\" ");

			fprintf( fcachefp,
				"7%s\t7form%s(%s.cache)\t%s\t%s\r\n",
				text, formpath, formpath,
				myhost, defaultport);
			
			strcat( arglist, argtmp);
			continue;
		}
	
		if ( streq( bufp, "Choose")) {
			choosenum++;
			if ( lname[0] != '\0') 
				writelink( lname, lpath, ltype, lhost, lport);
			if ( ( cp = strchr( text, '\t')) != NULL ) {
				*cp++ = '\0';
				sprintf( argtmp, "\"%s\" ", cp);
			}
			else
				strcpy( argtmp, "\"\" ");

			fprintf( fcachefp,
			    "1%s\t1choose%s%d.choose(%s.cache)\t%s\t%s\r\n",
				text, formpath, choosenum, formpath,
				myhost, defaultport);
			
			strcat( arglist, argtmp);
			continue;
		}
		if ( streq( bufp, "Choices"))
			dochoice( text, cpath, formpath, choosenum);

		if ( streq( bufp, "Done")) {
			if ( lname[0] != '\0')
				writelink( lname, lpath, ltype, lhost, lport);

			fprintf( fcachefp,
				"%c%s\texec%c:%s:%s(%s.cache)\t%s\t%s\r\n",
				exectype, text, exectype, arglist, formpath,
				formpath, myhost, defaultport);
			continue;
		}
		if ( streq( bufp, "Name")) {
			if ( *lname != '\0')
				writelink( lname, lpath, ltype, lhost, lport);
			copy( lname, text);
			continue;
		}
		if ( streq( bufp, "Path"))
			copy( lpath, text);
		if ( streq( bufp, "Type"))
			copy( ltype, text);
		if ( streq( bufp, "Host"))
			copy( lhost, text);
		if ( streq( bufp, "Port"))
			copy( lport, text);
		if ( streq( bufp, "Returntype"))
			exectype = *text;
	}	

	fprintf( stderr, "I think you forgot the \"Endform=\" line\n");
	fprintf( stderr, "for the form:    %s\n", formname);

	fclose( fcachefp);
}

static void
dochoice( text, cpath, formpath, chnum)
char	*text,
	*cpath,
	*formpath;
int	chnum;
{
	FILE	*chfp;

	char	*cp,
		*cp2,
		choosepath[MAXLEN];

	cp2 = strrchr( formpath, '/');

	strcpy( choosepath, cpath);
	cp = strrchr( choosepath, '/');
	sprintf( cp, "%s%d.choose", cp2, chnum);

	if ( (chfp = fopen( choosepath, "w")) == (FILE *) NULL ) {
		fprintf( stderr, "Can't open %s\n", choosepath);
		exit( 2);
	}

	printf( "Writing form choose file %s\n", choosepath);
	cp = text;
	while ( cp != NULL ) {
		cp = strchr( text, '\t');
		if ( cp != NULL)
			*cp++ = '\0';
		fprintf( chfp, "1%s\t1form%s(%s%d.choose)\t%s\t%s\r\n",
			text, formpath, formpath,
				chnum, myhost, defaultport);
		if ( cp != NULL)
			text = cp;
	}
}



/* chop( line)  Cut out CRLF at end of line */

void
chop( line)
char *line;
{
	char	*p;

	if ( *line == '\0')
		return;
	if ( (p = strchr( line, '\n')) == (char *) NULL )
		return;
	if ( *--p != '\r')
		p++;
	*p = '\0';
}


static void
writelink( ln, lp, lt, lh, lprt)
char	*ln,
	*lp,
	*lt,
	*lh,
	*lprt;
{
	if ( *lt == '\0') {
		*lt++ = *lp;
		*lt-- = '\0';
	}
	if ( *lh == '\0')
		strcpy( lh, myhost);
	if ( *lprt == '\0')
		strcpy( lprt, defaultport);

	fprintf(fcachefp, "%s%s\t%s\t%s\t%s\r\n", lt, ln, lp, lh, lprt);
	*ln = *lt = *lp = *lh = *lprt = '\0';
}