/*
    Gn: A Server for the Internet Gopher Protocol(*).
    File: mkcache/mkcache.c
    Version 0.6
    
    Copyright (C) 1993  <by John Franks>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    (*) Gopher is a registered trademark of the Univ. of Minn.
*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "mkcache.h"
#include "reg.h"

char	item[MAXLEN],
	host[MAXLEN],
	type[MAXLEN],
	port[MAXLEN],
	separator[MAXLEN],
	section[MAXLEN],
	path[MAXLEN],
	mfpath[MAXLEN],
	cfpath[MAXLEN];

static void	mkcache(),
		writeitem(),
		cache_composite();


main( argc, argv)
int	argc;
char	*argv[];
{
	init( argc, argv);
	mkcache( mfpath, cfpath);
}

static void
mkcache( mpath, cpath)
char	*mpath,
	*cpath;
{
	FILE	*mfp,
		*cfp;

	char	*cp,
		*text,
		*bufp,
		mbuf[MAXLEN];

	if ( (mfp = fopen( mpath, "r")) == (FILE *) NULL ) {
		fprintf( stderr, "Can't open %s\n", mpath);
		exit( 2);
	}

	if ( (cfp = fopen( cpath, "w")) == (FILE *) NULL ) {
		fprintf( stderr, "Can't open %s\n", cpath);
		exit( 2);
	}

	printf( "Writing cache file %s\n", cpath);

	while ( bufp = get_next_line(mbuf, MAXLEN, mfp)) {
		cp = strchr( bufp, '=');
		*cp++ = '\0';
		text = cp;

		if ( streq( bufp, "Name")) {
			if ( item[0] != '\0')
				writeitem( cfp, mpath, cpath);
			copy( item, text);
		}
		if ( streq( bufp, "Form")) {
			if ( item[0] != '\0')
				writeitem( cfp, mpath, cpath);
			doform( text, cpath, mfp, cfp);
			continue;
		}
		if ( streq( bufp, "Path"))
			copy( path, text);

		if ( streq( bufp, "Type"))
			copy( type, text);

		if ( streq( bufp, "Host"))
			copy( host, text);

		if ( streq( bufp, "Port"))
			copy( port, text);

		if ( streq( bufp, "Separator"))
			strcpy( separator, text);

			/* Use strcpy to allow trailing whitespace 
			  and tabs in separator and section regexps */

		if ( streq( bufp, "Section"))
			strcpy( section, text);

	}	
	if ( item[0] != '\0' )
		writeitem( cfp, mpath, cpath);
	fclose( cfp);
	fclose( mfp);
}


static void 
writeitem( fp, mpath, cpath)
FILE	*fp;
char	*mpath,
	*cpath;
{
	struct stat stat_buf;

	char	*cp1,
		*cp2,
		type1,
		tmppath[MAXLEN],
		mnewpath[MAXLEN],
		cnewpath[MAXLEN];

	if ( *path == '\0' ) {
		fprintf( stderr, 
			"Warning: Path field for item %s is empty.\n", item);
	}
	strcpy( tmppath, path);

	if ( *host == '\0' || streq( host, "+"))
		strcpy( host, myhost);
	if ( *port == '\0' || streq( port, "+"))
		strcpy( port, defaultport);
	if ( *type == '\0' ) 
		type1 = tmppath[0];
	else
		type1 = *type;

	/* If we have a composite (structured) file write its cache now */
	if ( (strncmp( path, "1m", 2) == 0) ||(strncmp( path, "7m", 2) == 0))
		if (separator[0] != '\0')
			cache_composite( path, cpath);
		else {
			printf( "\tWarning: item %s has no Separator\n", path);
			printf( "\tPresumably %s..cache has been made\n",path);
		}
	fprintf( fp, "%c%s\t%s\t%s\t%s\r\n",
				type1, item, path, host, port);

	item[0] = path[0] = host[0] = port[0] = type[0] = '\0';
	separator[0] = section[0] = '\0';

	if (recurse && (type1 == '1') && (tmppath[1] != 'm')) {

		if ( tmppath[ strlen(tmppath) - 1] == '/')
			tmppath[ strlen(tmppath) - 1] = '\0';
		if ( ( cp1 = strchr( tmppath, '/')) == NULL)
			return;

		/*
		 * Kill any '/' at the end of a path.  If that was the
                 * only '/' or there were none to begin with, then the path
		 * is the root or something weird and we don't want
		 * to recurse so we return.
		 */

		/* Next build new menu and cache path names and recurse */

		strcpy( mnewpath, mpath);
		cp1 = strrchr( mnewpath, '/');
		cp2 = strrchr( tmppath, '/');
		sprintf( cp1, "%s/%s", cp2, mfname);

                strcpy( cnewpath, cpath);
		cp1 = strrchr( cnewpath, '/');
		cp2 = strrchr( tmppath, '/');
		sprintf( cp1, "%s/%s", cp2, cfname);

		/* Check menu file exists, otherwise just return */
		if ( stat( mnewpath, &stat_buf) == 0 )
			mkcache( mnewpath, cnewpath);
	}
}


static void
cache_composite( selector, cpath)
char	*selector,
	*cpath;
{

	FILE	*cachefp,
		*compfp;

	struct regprog	*sep_rp,
			*title_rp;

	int	len,
		firsttime = TRUE,
		titlefound = FALSE;

	long	count = 0L,
		start = 0L,
		end = 0L;

	char	*cp,
		*cp2,
		compcache[MAXLEN],
		compfile[MAXLEN],
		default_title[MAXLEN],
		ltitle[MAXLEN],
		ttitle[MAXLEN],
		*linep,
		linebuf[MAXLEN];


	/* 
	 * selector (from Path= field in menu file) has the form 
	 *      1m/path/to/file   or 7m/path/to/file
	 * The separator and section arrays contain the regular 
	 * expressions to be used to separate the file into sections
	 * and make a menu item for each.
	 */


	strcpy( compfile, cpath);
	cp = strrchr( compfile, '/');
	cp2 = strrchr( selector, '/');
	strcpy( cp, cp2);
	sprintf( compcache, "%s.%s", compfile, cfname);

	if ( ((sep_rp = regcomp( separator)) == NULL) ||
			((title_rp = regcomp( section)) == NULL)) {
		fprintf( stderr, "Syntax error in %s\n or in %s\n",
			separator, section);
		exit( 2);
	}

	if ( (cachefp = fopen( compcache, "w")) == (FILE *) NULL ) {
		fprintf( stderr, "Warning: can't open %s\n", compcache);
		fprintf( stderr, "Skipping file %s.\n", compfile);
		return;
	}
	if ( (compfp = fopen( compfile, "r")) == (FILE *) NULL ) {
		fprintf( stderr, "Warning: can't open %s\n", compfile);
		fprintf( stderr, "Skipping file %s.\n", compfile);
		unlink( compcache);
		return;
	}


	printf( "Writing composite cache file %s\n", compcache);

	ltitle[0] = '\0';
	while ( fgets( linebuf, MAXLEN, compfp)) {
		len = strlen( linebuf);
		chop( linebuf);

		if ( regfind( sep_rp, linebuf) ) {
			end = count;
			sprintf( default_title, " Range: %d--%d", start, end);
			if ( ! firsttime) {
				copy( ttitle, ltitle);
				fprintf( cachefp, "0%s\tR%d-%d-%s\t%s\t%s\r\n",
					( *ttitle ? ttitle : default_title),
					start, end, selector, host, port);
			}
			start = count;
			*ltitle = '\0';
			firsttime = titlefound = FALSE;
		}

		if ( (! titlefound) && regfind( title_rp, linebuf)) {
			strcpy( ltitle, regrp( 0));
			titlefound = TRUE;
		}

		count += len;
	}	
	end = count;
	copy( ttitle, ltitle);
	fprintf( cachefp, "0%s\tR%d-%d-%s\t%s\t%s\r\n",
		( *ttitle ? ttitle : default_title),
		start, end, selector, host, port);
	fclose( cachefp);
	fclose( compfp);
}


/*
 * copy( s1, s2) copies string s2 to s1 expanding tabs to 8 space boundaries
 * and removing any trailing spaces.
 */

void
copy( s1, s2)
char	*s1,
	*s2;
{
	char	*cp,
		*cp2;
	int	i;


	cp = s2;
	cp2 = s1;
	while ( *cp){
		if ( *cp == '\t') {
			for ( i = strlen(s1) % 8; i < 8; i++ )
				*cp2++ = ' ';
			cp++;
		} else {
			*cp2++ = *cp++;
		}
	}
	*cp2 = '\0';
	if ( cp2 > s1)
		cp2--;
	while ( (*cp2 == ' ') && ( cp2 >= s1 ))
		*cp2-- = '\0';
}

/* 
 * Read in line, skip lines with no "=" in them, deal with comments (#),
 * get rid of leading whitespace.
 */
			
char
*get_next_line( buf, len, fp)
char	*buf;
int	len;
FILE	*fp;
{
	char	*cp,
		*bufp;

	while ( bufp = fgets( buf, len, fp)) {
		chop( bufp);
		if ( (cp = strchr( bufp, '#')) != NULL) {
			if ( cp == bufp ) {
				*cp = '\0';
			} else if ( *--cp == '\\')
				strcpy( cp, cp+1);
		}
		if ( (cp = strchr( bufp, '=')) == NULL)
			continue;
		while ( (*bufp == '\t') || (*bufp == ' '))
			bufp++;
		break;
	}
	return bufp;
}
	